/* *******************************************************************************
 *  Copyright (C) 2014-2021 Mehmet Gunce Akkoyun Can not be copied and/or
 *	distributed without the express permission of Mehmet Gunce Akkoyun
 *	This library is a combined book of enviroment sensor library.
 *
 *	Library				: B902AA Device Library
 *	Code Developer		: Mehmet Gunce Akkoyun (akkoyun@me.com)
 *	Revision			: 1.0.0
 *	Relase				: 19.08.2021
 *
 *********************************************************************************/

#include "B902AA.h"

// Hardware Functions
void STF_B902AA::HardwareDiagnostic(void) {

	// Control for I2C MUX (Connected to Main I2C Line)
	Wire.beginTransmission(0x77); if ((Wire.endTransmission()) == 0) Diagnostic.Mux = true;

	// Control for Temperature Sensor
	if (Multiplexer(_TEMPERATURE_)) {
		
		// Connect Address
		Wire.beginTransmission(0x40);
		
		// Control for Connection
		if ((Wire.endTransmission()) == 0) {
		
			// Set Variable
			Diagnostic.Temperature = true;
		
		} else {

			// Set Variable
			Diagnostic.Temperature = false;

		}
		
	} else {

		// Set Variable
		Diagnostic.Temperature = false;

	}

	// Control for Internal LCD
	if (Multiplexer(_INTLCD_)) {
		
		// Connect Address
		Wire.beginTransmission(0x27);
		
		// Control for Connection
		if ((Wire.endTransmission()) == 0) {
		
			// Set Variable
			Diagnostic.INTLCD = true;
		
		} else {

			// Set Variable
			Diagnostic.INTLCD = false;

		}
		
	} else {

		// Set Variable
		Diagnostic.INTLCD = false;

	}

	// Control for External LCD
	if (Multiplexer(_EXTLCD_)) {
		
		// Connect Address
		Wire.beginTransmission(0x27);
		
		// Control for Connection
		if ((Wire.endTransmission()) == 0) {
		
			// Set Variable
			Diagnostic.EXTLCD = true;
		
		} else {

			// Set Variable
			Diagnostic.EXTLCD = false;

		}
		
	} else {

		// Set Variable
		Diagnostic.EXTLCD = false;

	}

	// Turn I2C Channel to 0
	Multiplexer(_OFF_);

	// Control for I2C Battery Gauge
	Wire.beginTransmission(0x36); if ((Wire.endTransmission()) == 0) Diagnostic.Gauge = true;

}

// LED Functions
void STF_B902AA::LED(uint8_t _LED, uint8_t _Color, bool _State) {

	// Color Selection
	switch (_Color) {
		
		case RED:

			// Control for LED
			if (_LED == LED1) {

				// Red Color
				if (_State) PORTF |= 0b00000001;
				if (!_State) PORTF &= 0b11111110;

			} else {
				
				// Red Color
				if (_State) PORTF |= 0b00001000;
				if (!_State) PORTF &= 0b11110111;

			}

			break;
		
		case GREEN:
			
			// Control for LED
			if (_LED == LED1) {

				// Green Color
				if (_State) PORTF |= 0b00000010;
				if (!_State) PORTF &= 0b11111101;

			} else {
				
				// Red Color
				if (_State) PORTF |= 0b00010000;
				if (!_State) PORTF &= 0b11101111;

			}

			break;
		
		case BLUE:
			
			// Control for LED
			if (_LED == LED1) {

				// Blue Color
				if (_State) PORTF |= 0b00000100;
				if (!_State) PORTF &= 0b11111011;

			} else {
				
				// Red Color
				if (_State) PORTF |= 0b00100000;
				if (!_State) PORTF &= 0b11011111;

			}

			break;
		
		default:
		
			// Turn All LED Off
			PORTD &= 0b10001111;
			
			break;
	}

}
void STF_B902AA::LED_Blink(uint8_t _LED, uint8_t _Color, uint8_t _Blink, int _Interval) {
	
	// Blink Loop
	for (uint8_t i = 0; i < _Blink; i++) {
		
		// Turn ON LED
		LED(_LED, _Color, true);
		
		// ON Interval
		delay(_Interval);
		
		// Turn OFF LED
		LED(_LED, _Color, false);
		
		// OFF Interval
		delay(_Interval);

	}
	
}

// I2C Functions
bool STF_B902AA::Multiplexer(uint8_t _Channel) {
	
	/******************************************************************************
	 *	Project		: TCA9546 I2C Multiplexer Function
	 *	Developer	: Mehmet Gunce Akkoyun (akkoyun@me.com)
	 *	Revision	: 01.00.00
	 *	Release		: 18.07.2021
	 ******************************************************************************/

	// Control for Channel
	if (_Channel >= 0 and _Channel <= 4 and Diagnostic.Mux == true) {
	
		// Control for Current Channel
		if (_Channel != _Multiplexer_Current_Channel) {
			
			// Connect I2C Multiplexer
			Wire.beginTransmission(0x77);

			// Change Channel
			if (_Channel == 0) Wire.write(0x00);	// Turn Multiplexer OFF
			if (_Channel == 1) Wire.write(0x01);	// Turn Multiplexer to CH1
			if (_Channel == 2) Wire.write(0x02);	// Turn Multiplexer to CH2
			if (_Channel == 3) Wire.write(0x04);	// Turn Multiplexer to CH3
			if (_Channel == 4) Wire.write(0x08);	// Turn Multiplexer to CH4

			// Close I2C Connection
			Wire.endTransmission();

			// Set Variable
			_Multiplexer_Current_Channel = _Channel;
			
			// Delay
			delay(20);

			// End Function
			return(true);

		} else {
			
			// End Function
			return(true);
			
		}
		
	} else {
		
		// End Function
		return(false);

	}
	
}

STF_B902AA B902AA;
